package blosum;

import blosum.*;
import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.net.URL;
import java.util.Vector;
import java.util.Enumeration;

/**
* @(#)Computation.java
* 
* Performes the computation and display, if necessary, a progress monitor.
*
* @author Pussini Marco, Marica Rigo
*/
public class Computation extends Thread {
	
	/**
     * Holds the computation result
     */
	public class Results {
		
		public final int SCORE;
		public final int OFFSET;
		public final int IDENTITY;
		public final int MAT;
		public final float IXY;
		public final float DXY;
		public final float DX;
		public final float DY;
		public final float NBS;
		public final Sequence FIRSTSEQ;
		public final Sequence SECONDSEQ;
		public final String COMPARESTR;
		
		public Results(Sequence first, Sequence second, Sequence subseq, int offset,int mat, ProteinMatrix pm, Matrixqij mq, Matrixbla mb) throws Exception{
			this.FIRSTSEQ = first;
			this.SECONDSEQ = second;
			this.IXY = Scores.mutualInfo(subseq, second, pm);
			this.OFFSET = offset;
			this.MAT = mat;
			this.DXY = Scores.divXY(pm, mq);
			this.DX = Scores.divX(subseq, mq);
			this.DY = Scores.divX(second, mq);
			this.NBS = this.IXY - this.DXY + this.DX + this.DY;
			this.SCORE = Scores.blosumScore(subseq, second, mb);
			this.IDENTITY = pm.getIdentity();      
			this.COMPARESTR = Scores.getCompareString(subseq, second, mb);
		}
		
		/**
	     * Checks if the current Result set is better than a given one
	     *
	     * @param res The Results to be compared with
	     */
		public Results getBest(Results res) {
			return ((this.NBS < res.NBS) ? res : this);
		}
		
	} //Results
	
	private Sequence first;
	private Sequence second;
	private ProgressMonitor monitor;
	private int mat;
	private String base;
	private Vector listeners = new Vector();
	private Results best;
	
	/**
	* Creates a new Computation for the specific input sequences
	*
	* @param parent The Component that owns the ProgressMonitor
	* @param s1 The first input Sequence
	* @param s2 The second input Sequence
	* @param mat The Blosum matrix index
	* @param base The base URL where search the .bla and .qij files
	*/
	public Computation(Component parent, Sequence s1, Sequence s2, int mat, URL base) {
		this.setPriority(Thread.MAX_PRIORITY); // Max priority to the computation
		this.mat = mat;
		this.base = base.toString();
		// The first sequence will be the longer one
		if (s2.getSequence().length() > s1.getSequence().length()) {
			first = s2;
			second = s1;
		} else {
			first = s1;
			second = s2;
		}
		// The difference of length between the two sequences
		int length = first.getSequence().length() - second.getSequence().length();
		monitor = new ProgressMonitor(parent, "Computation in progress...", "", 0, length);
		// If the computation does non terminate within one second, the progress monitor will popup
		monitor.setMillisToDecideToPopup(500);
		monitor.setMillisToPopup(1000);
	}
	
	/**
	* Starts the computation
	*/
	public void startComputation() {
		this.start();
	}
	
	public void run() {
		int length = first.getSequence().length() - second.getSequence().length() + 1;
		int i = 0;
		best = null;
		monitor.setProgress(0); // reset the monitor
		try {
			// loads the two auxiliary matrix from a .qij and .bla files
			Matrixqij mq = new Matrixqij(new URL(base + "matrix/blosum"+ mat +".qij"));
			Matrixbla mb = new Matrixbla(new URL(base + "matrix/blosum"+ mat +".bla"));
			// computes the best sequence allignment until the end or the canceling of the user
			for (i = 0; i < length && !monitor.isCanceled(); i++) {
				Sequence subseq = first.subsequence(i, second.getSequence().length()); // subsequence
				ProteinMatrix pm = new ProteinMatrix(subseq, second);
				Results current = new Results(first, second, subseq, i,mat, pm, mq, mb);
				best = ((best == null) ? current : best.getBest(current)); // mantains the best allignment
				monitor.setProgress(i);
			}
			monitor.setProgress(length); // be sure to display the end
		} catch (Exception ex) { // error!
			// we close the monitor and display the error message
			monitor.close();
			JOptionPane.showMessageDialog(null, ex.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
		}
		// notifies the listener and passes to them the results (if the computation has not been canceled)
		this.fireEvent((i >= length) ? best : null);
	}
	
	/**
	* Registers a new ComputationListener
	*
	* @param listener The ComputationListener to register.
	*/
	public void addComputationListener(ComputationListener listener) {
		synchronized (listeners) {
			listeners.add(listener);
		}
	}
	
	/**
	* Removes a ComputationListener from the listeners list
	*
	* @param listener The ComputationListener to remove.
	*/
	public void removeComputationListener(ComputationListener listener) {
		synchronized (listeners) {
			listeners.remove(listener);
		}
	}
	
	/**
     * Notifies the listeners of the computation's termination
     *
     * @param results The Results of the computation or null if the computation has been cancelled
     */
	private void fireEvent(Results results) {
		synchronized (listeners) {
			Enumeration e = listeners.elements();
			while (e.hasMoreElements()) {
				((ComputationListener)e.nextElement()).computationTerminated(results);
			}
		}
	}
	
}
