package blosum;

/**
 * @(#)Scores.java
 * 
 * Scores provide a set of routines for the computation of the metrics:
 *	- Mutual Information
 *	- Blosum Score
 *	- Divergence
 *
 * @author Pussini Marco, Marica Rigo
 */
public class Scores {
	
	/**
	 * Calculates the Mutual Information of the two sequences given.
	 *
	 * @param s1 First Sequence
	 * @param s2 Second Sequence
	 * @param mp The MatrixProtein containing the pairs frequences
	 *
	 * @return The Mutual Information
	 */
	public static float mutualInfo(Sequence s1, Sequence s2, ProteinMatrix pm) {
		double minfo = 0;
		for (int i = 1; i < 21; i++) {
			for (int j = 1; j < 21; j++) {
				double fxy = pm.getFrequence(i, j);
				double fxfy = s1.getFrequence(i)*s2.getFrequence(j);
				if (fxy > 0 && fxfy > 0) {
					minfo += (fxy * Math.log(fxy / fxfy));
				}
			}
		}
		minfo /= Math.log(2);
    return (float)minfo;
	}

	/**
	 * Calculates the Informational Divergence between the pairs frequencies and
	 * the pairs probabilities
	 *
	 * @param mp The MatrixProtein that holds the pairs frequencies
	 * @param mq The MatrixQij that holds the pairs Probabilities
	 *
	 * @return The Informational Divergence.
	 */
	public static float divXY(ProteinMatrix pm, Matrixqij mq) {
		double divxy = 0;
		for (int i = 1; i < 21; i++) {	
			for (int j = 1; j < 21; j++) {
				double fxy = pm.getFrequence(i, j);
				double pxy = mq.getProbability(i, j);
				if (fxy > 0 && pxy > 0) {
					divxy += (fxy * (Math.log(fxy / pxy)));
				}
		    }
	    }
		divxy /= Math.log(2);
	    return (float)divxy;
	}

	/**
	 * Calculates the Informational Divergence between the amino acids frequencies and
	 * probabilities into a given Sequence
	 *
	 * @param s The Sequence that holds the frequencies
	 * @param mq The MatrixQij that holds the probabilities
	 *
	 * @return The Informational Divergence
	 */
	public static float divX(Sequence s, Matrixqij mq) {
		double divx = 0;
		for (int i = 1; i < 21; i++) {
			float fx = s.getFrequence(i);
			float px = mq.getMarginal(i);
			if (fx > 0 && px > 0) {
				divx += (fx * Math.log(fx / px));
		    }
	    }
	    divx /= Math.log(2);
	    return (float)divx;
	}

	/**
	 * Calculates the BLOSUM score associated to the Sequences given
	 *
	 * @param s1 First Sequence
	 * @param s2 Second Sequence
	 * @param mb The MatrixBla that contains the marginal scores of the pairs
	 *
	 * @return The BLOSUM score
	 *
	 * @throws
	 */
	public static int blosumScore(Sequence s1, Sequence s2, Matrixbla mb) throws Exception{
		String x = s1.getSequence();
		String y = s2.getSequence();
		int max = Math.min(x.length(), y.length());
		int score = 0;
		for (int i = 0; i < max; i++) {
			score += mb.getScore(x.charAt(i), y.charAt(i));
    }
		return score;
	}

  	/**
	 * Calculates the BLOSUM score associated to the Sequences given
	 *
	 * @param s1 First Sequence
	 * @param s2 Second Sequence
	 * @param mb The MatrixBla that contains the marginal scores of the pairs
	 *
	 * @return The BLOSUM score
	 *
	 * @throws
	 */
	public static String getCompareString(Sequence s1, Sequence s2, Matrixbla mb) throws Exception{
		String x = s1.getSequence();
		String y = s2.getSequence();
		int max = Math.min(x.length(), y.length());
		String s = "";
		for (int i = 0; i < max; i++) {
      		char cx = x.charAt(i);
      		char cy = y.charAt(i);
      		if (cx == cy) {
       			s += cx;
      		} else {
          		s += ((mb.getScore(cx, cy) > 0 ) ? "+" : " ");
      		}
    	}
		return s;
	}
	
}