package blosum;

import java.util.Vector;
import blosum.utils.InputUtils;
import java.net.URL;
import java.io.File;

/**
 * @(#)Sequence.java
 *
 * Keeps track of all the information concerning the Protein Sequence such that the String sequence,
 * the protein frequency into the sequence and a set of helpful routines
 *
 * @author Pussini Marco, Marica Rigo
 */
public class Sequence {
	
	private String name;
	private String seq;
	private int[] occurences;
	private int aminos;

	/**
	 * Creates a new Sequence from two strings
	 *
	 * @param name The name of the sequence
	 * @param sequence The sequence of amino acids
	 *
	 * @throws
	 */
	public Sequence(String name, String sequence) throws Exception{
		this.name = name;
		this.seq = sequence.toUpperCase();
		calcOccurences();
	}

	/**
	 * Creates a new Sequence from a Vector of data.
	 *
	 * @param data The Vector of String containing, in order:
	 *	- the name of the Sequence beginning with '>'
	 *	- the amino acid sequence divided into one or more rows
	 *
	 * @throws
	 */
	public Sequence(Vector data) throws Exception{
		if (data.size() < 2) {
			throw new Exception("Insufficient data.");
	    }
		String row = (String)data.firstElement();
		if (row.charAt(0) != '>') {
			throw new Exception("Unrecognized sequence name: \"" +
				row.substring(0,Math.min(row.length(), 10)) + "\".");
		} else {
			this.name = row;
			this.seq = "";
			data.remove(0);
		}
		do {
			row = (String)data.firstElement();
			if (row.length() > 0 && row.charAt(0) != '>') {
				this.seq += row;
				data.remove(0);
		    }
	    } while(data.size() > 0 && (!(row.length() > 0) || row.charAt(0) != '>'));
	    if (this.seq.length() < 1) {
	 		throw new Exception("Insufficient data.");
	    }
      this.seq = this.seq.toUpperCase();
	    calcOccurences();
	}

	/**
	 * Method getName
	 *
	 * @return The name of the Sequence
	 */
	public String getName() {
		return this.name;
	}

	/**
	 * Method getSequence
	 *
	 * @return The amino acid string of the Sequence
	 */
	public String getSequence() {
		return this.seq;
	}

	/**
	 * Gives the frequency of the amino acid into the Sequence
	 *
	 * @param idx The index (0<=idx<=19) of the amino acid
	 *
	 * @return The frequence of the amino acid
	 */
	public float getFrequence(int idx) {
		if (idx<1 || idx>20) {
			return 0;
	    } else {
			return ((float)this.occurences[idx-1])/this.aminos;
	    }
	}

	/**
	 * Gives the subsequence of a specified length of the Sequence
	 *
	 * @param begin Begin index (0<=begin<=seq.length())
	 * @param length Length of the subsequence (0<=length<=seq.length()-begin)
	 *
	 * @return The subsequence
	 *
	 * @throws
	 */
	public Sequence subsequence(int begin, int length) throws Exception {
		if (begin+length > this.seq.length()) {
			throw new Exception("Illegal arguments");
		} else {
			Sequence newSeq = new Sequence(this.name, this.seq.substring(begin, (begin+length)));
			return newSeq;
		}
	}

	/**
	 * Calculates the occurences of the amino acids into the Sequence
	 *
	 * @throws
	 */
	private void calcOccurences() throws Exception{
		this.aminos = 0;
		this.occurences = new int[20];
		for (int i = 0; i < this.seq.length(); i++) {
			int idx = AminoAcid.indexOfAmin(this.seq.charAt(i));
			if (idx > 0) {
				this.aminos++;
				this.occurences[idx-1] = this.occurences[idx-1] + 1;
		    }
	    }
	}
	
	/**
	 * Extracs the id code from the sequence name
	 *
	 * @return The code identifying the Sequence
	 */
	public String getCode() {
        int position2 = name.lastIndexOf("|");
        int position1 = name.lastIndexOf("|",position2-1);
        if (position1 >= 0 && position2 >= 0 && position1 < position2-1)
                return name.substring(position1+1,position2);
        else    return "";
	}
}

