package blosum.utils;

import blosum.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.net.URL;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.Enumeration;

/**
* @(#)SequenceLoader.java
*
* A JPanel that implements a loader of FASTA protein sequences.
* The esequences can be loaded from a text file (specified from the user using a FileOpenDialog) or
* from a manual insertion of the user into a InputDialog.
*
* @author Pussini Marco, Marica Rigo
*/
public class SequenceLoader extends JPanel{

	/**
     * InputDialog implements a dialog that allows the user to enter manually the FASTA protein sequence,
     * instead of the loading of a file.
     */
	private class InputDialog extends JDialog {
		
		private JTextArea area = new JTextArea(20,50); // Here the user enters the text
		private JButton ok = new JButton("Accept");
		private JButton cancel = new JButton("Cancel");
		private JLabel message = new JLabel("<html>Please, insert the sequence in FASTA format.<br>" +
		"NOTE: To use files instead of manual insertion, please accept the certificate next time.</html>");
		private Vector result;
		private JScrollPane scroll;
		
		/**
	     * Creates a new InputDialog with a specified owner
	     *
	     * @param The Frame that owns the dialog
	     */
		public InputDialog(Frame owner) {
			super(owner,"Insert FASTA sequence", true);
			this.getContentPane().setLayout(new BorderLayout());
			
			area.setFont(new Font("Monospaced", Font.PLAIN,12));
			area.setWrapStyleWord(false);
			area.setLineWrap(true);
			area.setMargin(new Insets(5,5,5,5));
			
			JPanel p = new JPanel(new BorderLayout(5,5));
			p.setOpaque(false);
			p.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
			p.setLayout(new BorderLayout(5,5));
			p.add(message, BorderLayout.NORTH);
			
			scroll = new JScrollPane(area, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
			JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
			p.add(scroll, BorderLayout.CENTER);
			
			JPanel p2 = new JPanel(new FlowLayout(FlowLayout.RIGHT));
			p2.setOpaque(false);
			ok.addActionListener(new ActionListener() { // listen to "OK"
				public void actionPerformed(ActionEvent e) {
					pressed(true);
				}
			});
			cancel.addActionListener(new ActionListener() { // listen to "CANCEL"
				public void actionPerformed(ActionEvent e) {
					pressed(false);
				}
			});
			p2.add(ok);
			p2.add(Box.createRigidArea(new Dimension(5,5)));
			p2.add(cancel);
			p.add(p2, BorderLayout.SOUTH);
			
			this.addWindowListener(new WindowAdapter() { // listen to the dialog closing
				public void windowClosing(WindowEvent e) {
					pressed(false);
					setVisible(false);
				}
			});
			this.getContentPane().add(p);
			this.pack();
		}
		
		/**
	     * Called outside the class to disply the dialog
	     *
	     * @return A Vector containing the rows of the typed text
	     */
		public Vector showDialog() {
			// This dialog is modal, so when it is visible the calling frame will be freezed until
			// we call setVisible(false)
			this.setVisible(true);
			return result;
		}
		
		/**
	     * Called when the OK button or the CANCEL button are pressed
	     *
	     * @param true in case of OK button, false otherwise
	     */
		private void pressed(boolean accept) {
			if (accept) { // the user validates his input
				result = new Vector();
				String seq = this.area.getText();
				StringTokenizer st = new StringTokenizer(seq,"\n",false);
				while (st.hasMoreTokens()) {
					result.add(st.nextToken());
				}
			} else {
				result = null;
			}
			this.setVisible(false); // Hides the dialog and frees the caller blocke into showDialog()
		}
		
	}// InputDialog

	private JTextField address;
	private JButton open;
	private JButton insert;
	private Sequence[] sequences;
	private JFileChooser chooser;
	private InputDialog input;
	
	private Vector listeners;
	
	/**
     * Creates a new SequenceLoader
     *
     * @param msg The message to display
     * @param seqs The number of sequences that the loader has to open
     */
	public SequenceLoader(String msg, int seqs) {
		super(new FlowLayout(FlowLayout.RIGHT));
		this.sequences = new Sequence[seqs];
		this.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		this.setOpaque(false);
		
		try { // Let's try to use a JFileChooser
			chooser = new JFileChooser();
			chooser.addChoosableFileFilter(new SequencesTxtFilter());
			if (seqs < 2) { // Set the appropriate file filter
				chooser.addChoosableFileFilter(new SequenceFaFilter());
				chooser.addChoosableFileFilter(new SequenceFastaFilter());
			}
			chooser.setAcceptAllFileFilterUsed(false);
		} catch (Exception ex) { // The user has not accepted the certificate
			chooser = null;
		}
		
		if (chooser != null) { // The user has accepted the certificate
			address = new JTextField(50);
			address.setText(msg);
			address.setEditable(false);
			open = new JButton("Load");
			open.addActionListener(new ActionListener() { // Listening to the load request
				public void actionPerformed(ActionEvent e) {
					openRequest();
				}
			});
			this.add(address);
			this.add(Box.createRigidArea(new Dimension(5,5)));
			this.add(open);
		} else { // The user has to insert the sequence manually
			input = new InputDialog(null);
			insert = new JButton("Insert Sequence");
			insert.addActionListener(new ActionListener() { // Listening to the insert request
				public void actionPerformed(ActionEvent e) {
					openRequest();
				}
			});
			JLabel l = new JLabel(msg, JLabel.RIGHT); // The message is displayed in a JLabel instead of the JTextField
			l.setOpaque(false);
			this.add(l);
			this.add(Box.createRigidArea(new Dimension(5,5)));
			this.add(insert);
		}
		listeners = new Vector(); // Initializes the list of the SequenceLoader listeners
	}
	
	/**
	* The user will specify the sequence (manually or choosing a file)
	*/
	private void openRequest() {
		// If we can use the chooser and (in the affermative case) if the user choosed a file...
		if (chooser != null && chooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
			File file = chooser.getSelectedFile(); // ...we open that file
			try {
				Vector v = new Vector();
				URL url = file.toURL();
				InputUtils.loadURL(url,v); // fills the vector with the sequence(s) rows
				if (loadSequences(v)) { // if the input loaded from the file is valid...
					this.address.setText(url.toString());
					fireEvent(); // ... we notify the listeners
				}
			} catch (Exception ex) { // In case of error we display it in a message dialog
				JOptionPane.showMessageDialog(null, "URL malformed!", "Error",
					JOptionPane.ERROR_MESSAGE);
			}
		} else if (chooser == null) { // The user has to inser the sequences manually using an InputDialog
			Vector v = input.showDialog();
			if (v != null && loadSequences(v)) { // if the user has accepted the input and it is valid...
				fireEvent(); // ...we notify the listeners
			}
		}
	}
	
	/**
	* Tries to create the sequence(s) from the loaded (inserted) input.
	*
	* @param v The Vector that contains the input
	*
	* @return True if the input is valid and the Sequence has been successfully created, False otherwise
	*/
	private boolean loadSequences(Vector v) {
		try { // we try to create all the requested Sequences
			for (int i = 0; i < this.sequences.length; i++) {
				this.sequences[i] = new Sequence(v);
			}
			return true;
		} catch (Exception ex) {
			this.sequences = new Sequence[sequences.length]; // clears the sequences
			// Shows the error
			JOptionPane.showMessageDialog(this,ex.getMessage(),"Error",JOptionPane.ERROR_MESSAGE);
			return false;
		}
	}

	/**
	* Verifies if the Environment can handle Files (if the user has accepted the certificate).
	*
	* @return True if a JFileChooser (or simply files) can be used.
	*/
	public static boolean canUseFiles()	{
		try {
			JFileChooser jfc = new JFileChooser();
			return true;
		} catch (Exception ex) {
			return false;
		}
	}

	/**
	* Adds a new SequenceLoaderListener to the listeners list
	*
	* @param listener The SequenceLoaderListenr to add.
	*/
	public void addSequenceLoaderListener(SequenceLoaderListener listener) {
		synchronized (listeners) {
			listeners.add(listener);
		}
	}

	/**
	* Removess a SequenceLoader from the listeners list
	*
	* @param listener The listeners to remove.
	*/
	public void removeSequenceLoaderListener(SequenceLoaderListener listener) {
		synchronized (listeners) {
			listeners.remove(listener);
		}
	}
	
	/**
	* Notifies all the registered listeners of the Sequence(s) loading
	*/
	private void fireEvent() {
		synchronized (listeners) {
			Enumeration e = listeners.elements();
			while (e.hasMoreElements()) {
				((SequenceLoaderListener)e.nextElement()).sequenceChoosed(this);
			}
		}
	}

	/**
	* Gives the last opened sequence(s)
	*
	* @return The Sequences requested or null if no sequence(s) has been succesfully created
	*/
	public Sequence[] getSequences() {
		return this.sequences;
	}

}
