package blosum.utils;

import java.awt.image.*;
import javax.imageio.*;
import javax.swing.*;
import java.awt.*;
import java.io.*;

/**
* @(#)SpectrumIstogram.java
*
* Implements an Istogram that shows the Blosum Spectrum.
*
* @author Pussini Marco, Marica Rigo
*/
public class SpectrumIstogram {
	
	private BufferedImage image;
	private int column_w, space_w, line_h, factor;
	
	/**
     * Creates a new SpectrumIstogram from the numeric values of the spectrum components
     *
     * @param column_w The width of the columns (in pixels)
     * @param space_w The space to left between the spectrum columns (in pixels)
     * @param line_h The height of the line that divides the positive and negative parts of the spectrum (in pixels)
     * @param factor The number of pixels displayed for 1 unit of the components values
     * @param nbs The Normalized Blosum Score
     * @param ixy The Mutual Information
     * @param dxy The Mutual Divergence
     * @param dx The divergence of the first sequence
     * @param dy The divergence of the second sequnece
     */
	public SpectrumIstogram(int column_w, int space_w, int line_h, int factor, float nbs, float ixy, float dxy, float dx, float dy) {
		this.initialize(column_w, space_w, line_h, factor, nbs, ixy, dxy, dx, dy);
	}
	
	public SpectrumIstogram(float nbs, float ixy, float dxy, float dx, float dy) {
		this.initialize(8, 8, 1, 20, nbs, ixy, dxy, dx, dy);
	}
	
	public SpectrumIstogram(float ixy, float dxy, float dx, float dy) {
		this.initialize(8, 8, 1, 20, (ixy - dxy + dx + dy), ixy, dxy, dx, dy);
	}
	
	private void initialize(int column_w, int space_w, int line_h, int factor, float nbs, float ixy, float dxy, float dx, float dy) {
		this.column_w = column_w;
		this.space_w = space_w;
		this.line_h = line_h;
		this.factor = factor;
		int[] values = new int[5];
		values[0] = Math.round(this.factor * nbs);
		values[1] = Math.round(this.factor * ixy);
		values[2] = -Math.round(this.factor * dxy); // The mutual Divergence has to be negative
		values[3] = Math.round(this.factor * dx);
		values[4] = Math.round(this.factor * dy);
		createBufferedImage(values); // creates the image of the istogram
	}
	
	/**
     * Creates a BufferedImage from the spectrum components values stored in an integer pixels values
     *
     * @param The amount of pixels of each component
     */
	private void createBufferedImage(int[] values) {
		// Determinates the total width of the image
		int w = (this.column_w * values.length) + (this.space_w * values.length);
		// Determinates the total height of the negative part of the spectrum
		int lower = Math.max(-values[0], -values[2]);
		// Determinates the total height of the positive part of the spectrum
		int upper = Math.max(values[0], Math.max(values[1], Math.max(values[3], values[4])));
		int height = upper+lower+this.line_h;
		image = new BufferedImage(w, height, BufferedImage.TYPE_3BYTE_BGR); // empty image created
		Graphics2D g = image.createGraphics();
		g.setBackground(Color.white); // back color is white
		g.clearRect(0,0,w,height);
		g.setColor(Color.black);
		g.fillRect(0, upper, w, this.line_h); // the line is black
		if (values[0] < 0) { // set the color for the first component (norm.BlosumScore)
			g.setColor(Color.red);
		} else {
			g.setColor(Color.blue);
		}
		int x = this.space_w / 2;
		for (int i = 0; i < values.length; i++) { // paints  each component to the image
			if (values[i] < 0) {
				g.fillRect(x, upper+line_h, this.column_w, -values[i]);
			} else {
				g.fillRect(x, upper-values[i], this.column_w, values[i]);
			}
			g.setColor(Color.black);
			x += (this.space_w + this.column_w);
		}
		g.dispose();
	}
	
	/**
     * Returns the Spectrum like an Icon 
     *
     * @return The Icon that represents the SpectrumIstogram
     */
	public Icon getIcon() {
		return new ImageIcon(this.image);
	}
	
	/**
     * Writes the SpectrumIstogram into a specific File. The format of the image is gived by the File extenision
     *
     * @param file The File where the Istogram has to be stored in.
     *
     * @return True if the extension has been recognized and the Istogram has been wrote to the File.
     */
	public boolean write(File file) {
		try {
			String name = file.getName();
			String extension = name.substring(name.lastIndexOf('.') + 1);
			return ImageIO.write(this.image, extension, file);
		} catch (Exception ex) {
			return false;
		}
	}
	
}